#ifndef TREE_NODE_HPP
#define TREE_NODE_HPP

#include "Body.hpp"
#include "Box.hpp"
#include "Tree.hpp"
#include "BarnesHutTree.hpp"
#include <cstdlib>
#include <vector>

namespace nbody {
	using namespace std;

	class Node {
		friend class Tree;
		friend class BarnesHutTree;
	protected:
		Box bb;
		vector<Body> bodies;
		vector<Body> refinements;
		Node* prev;
		Node* next;
		Node* nextSibling;
		Node* prevSibling;
		Node* parent;
		Node* afterSubtree;
		bool leaf;
		Tree* tree;
		Body representative;
	public:
		Node(Tree* tree);
		virtual ~Node();
		virtual bool isSplitable();
		virtual void extendBBforBodies();
		virtual void extendBBtoCube();
		virtual Box getBB();
		virtual vector<Body> getBodies();
		virtual void insertBefore(Node* node);
		virtual void insertAfter(Node* node);
		virtual void unlink();
		virtual void update();
		virtual double getL();
		virtual bool isCorrect();
		virtual void print();
		virtual bool sufficientForBody(Body body);
	};
}

#endif
